--------------------------------------General Notes------------------------------------------------
--IPchecksum value must be calculated manually, then hard-coded.

--EN_H pin can be tied hi if clock is 20mHz.  My circuit is 40MHz, so I used an external
--divide-by-two component that feeds EN_H
	
--STARTSENDING is a hardware input in this version.  A packet is sent when a one-clock
--wide pulse is recieved on this input.

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity TENBASET_TXD_RANDY_SEND_INPUT is
	 port(
		 CLOCK_R : in STD_LOGIC;
		 CLEAR_L : in STD_LOGIC;
		 EN_H : in STD_LOGIC;
		 STARTSENDING : in STD_LOGIC;
		 Ethernet_TDp : out STD_LOGIC;
		 Ethernet_TDm : out STD_LOGIC
	     );
end TENBASET_TXD_RANDY_SEND_INPUT;

architecture TENBASET_TXD_RANDY_SEND_INPUT of TENBASET_TXD_RANDY_SEND_INPUT is

SIGNAL IPSOURCE_1: STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL IPSOURCE_2: STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL IPSOURCE_3: STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL IPSOURCE_4: STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL IPDESTINATION_1:  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL IPDESTINATION_2:  STD_LOGIC_VECTOR(7 DOWNTO 0); 
SIGNAL IPDESTINATION_3:  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL IPDESTINATION_4:  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL PHYSICALADDRESS_1:  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL PHYSICALADDRESS_2:  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL PHYSICALADDRESS_3:  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL PHYSICALADDRESS_4:  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL PHYSICALADDRESS_5:  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL PHYSICALADDRESS_6:  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL RDADDRESS: STD_LOGIC_VECTOR (6 DOWNTO 0);
SIGNAL PKT_DATA: STD_LOGIC_VECTOR (7 DOWNTO 0);
SIGNAL SHIFTCOUNT: STD_LOGIC_VECTOR (3 DOWNTO 0);
SIGNAL SENDINGPACKET: STD_LOGIC;
SIGNAL TEMP_XHDL3: STD_LOGIC_VECTOR (3 DOWNTO 0); 
SIGNAL READRAM: STD_LOGIC;
SIGNAL TEMP_XHDL4: STD_LOGIC_VECTOR (6 DOWNTO 0);
SIGNAL SHIFTDATA: STD_LOGIC_VECTOR (7 DOWNTO 0);
SIGNAL TEMP_XHDL5: STD_LOGIC_VECTOR (7 DOWNTO 0);
SIGNAL CRC: STD_LOGIC_VECTOR (31 DOWNTO 0);
SIGNAL CRCFLUSH: STD_LOGIC;
SIGNAL CRCINIT: STD_LOGIC;
SIGNAL TEMP_XHDL6: STD_LOGIC;
SIGNAL CRCINPUT: STD_LOGIC;
SIGNAL CRCINPUTVEC: STD_LOGIC_VECTOR (31 DOWNTO 0);
SIGNAL CRCINPUTVEC_AND_X04C11DB7: STD_LOGIC_VECTOR (31 DOWNTO 0);
SIGNAL TEMP_XHDL7: STD_LOGIC_VECTOR (31 DOWNTO 0);
SIGNAL LINKPULSECOUNT: STD_LOGIC_VECTOR (17 DOWNTO 0);
SIGNAL TEMP_XHDL8: STD_LOGIC_VECTOR (17 DOWNTO 0);
SIGNAL LINKPULSE: STD_LOGIC;
SIGNAL SENDINGPACKETDATA: STD_LOGIC;
SIGNAL IDLECOUNT: STD_LOGIC_VECTOR (2 DOWNTO 0);
SIGNAL TEMP_XHDL9: STD_LOGIC;
SIGNAL DATAOUT: STD_LOGIC;
SIGNAL QO: STD_LOGIC;
SIGNAL TEMP_XHDL10: STD_LOGIC;
SIGNAL QOE: STD_LOGIC;
SIGNAL TEMP_XHDL11: STD_LOGIC;
SIGNAL TEMP_XHDL12: STD_LOGIC;
SIGNAL Ethernet_TDp_xhdl1: STD_LOGIC;
SIGNAL Ethernet_TDm_xhdl2: STD_LOGIC;
SIGNAL IPCHECKSUM3 : STD_LOGIC_VECTOR (31 DOWNTO 0);
SIGNAL TEMPRANDY1 : STD_LOGIC; 

SIGNAL COUNTER : STD_LOGIC_VECTOR (23 DOWNTO 0);

BEGIN 
	
IPSOURCE_1 <= X"0A"; --10d
IPSOURCE_2 <= X"0A"; --10d
IPSOURCE_3 <= X"10"; --16d
IPSOURCE_4 <= X"00"; --0d

IPDESTINATION_1	<= X"0A"; --10d 
IPDESTINATION_2	<= X"0A"; --10d
IPDESTINATION_3	<= X"10"; --16d
IPDESTINATION_4	<= X"EC"; --236d

PHYSICALADDRESS_1 <= "00000000";
PHYSICALADDRESS_2 <= "00001011";
PHYSICALADDRESS_3 <= "11011011";
PHYSICALADDRESS_4 <= "11000101";
PHYSICALADDRESS_5 <= "10110011";
PHYSICALADDRESS_6 <= "00000101";
	
Ethernet_TDp <= Ethernet_TDp_xhdl1;
Ethernet_TDm <= Ethernet_TDm_xhdl2;

--Note: IPCHECKSUM3 must be manually calculated, see Verilog equivalent
--for details!  
IPCHECKSUM3 <= X"000005C0";
		
PROCESS (CLOCK_R,CLEAR_L)
BEGIN
	IF CLEAR_L = '0' THEN
		PKT_DATA <= (OTHERS => '0');
	ELSIF RISING_EDGE (CLOCK_R) THEN
		IF EN_H = '1' THEN
		CASE RDADDRESS IS
			--ETHERNET PREAMBLE
			WHEN "0000000" =>
			PKT_DATA <= X"55";
			WHEN "0000001" =>
			PKT_DATA <= X"55";
			WHEN "0000010" =>
			PKT_DATA <= X"55";
			WHEN "0000011" =>
			PKT_DATA <= X"55";
			WHEN "0000100" =>
			PKT_DATA <= X"55";
			WHEN "0000101" =>
			PKT_DATA <= X"55";
			WHEN "0000110" =>
			PKT_DATA <= X"55";
			WHEN "0000111" =>
			PKT_DATA <= X"D5";
			
			--ETHERNET HEADER 
			
			WHEN "0001000" =>
			PKT_DATA <= PHYSICALADDRESS_1;
			WHEN "0001001" =>
			PKT_DATA <= PHYSICALADDRESS_2;
			WHEN "0001010" =>
			PKT_DATA <= PHYSICALADDRESS_3;
			WHEN "0001011" =>
			PKT_DATA <= PHYSICALADDRESS_4;
			WHEN "0001100" =>
			PKT_DATA <= PHYSICALADDRESS_5;
			WHEN "0001101" =>
			PKT_DATA <= PHYSICALADDRESS_6;
			WHEN "0001110" =>
			PKT_DATA <= X"00";
			WHEN "0001111" =>
			PKT_DATA <= X"12";	 
			WHEN "0010000" =>
			PKT_DATA <= X"34";
			WHEN "0010001" =>
			PKT_DATA <= X"56";
			WHEN "0010010" =>
			PKT_DATA <= X"78";
			WHEN "0010011" =>
			PKT_DATA <= X"90";
			
			--IP HEADER
			WHEN "0010100" =>
			PKT_DATA <= X"08";
			WHEN "0010101" =>
			PKT_DATA <= X"00";	 
			WHEN "0010110" =>
			PKT_DATA <= X"45";
			WHEN "0010111" =>
			PKT_DATA <= X"00";
			WHEN "0011000" =>
			PKT_DATA <= X"00";
			WHEN "0011001" =>
			PKT_DATA <= X"2E";
			WHEN "0011010" =>
			PKT_DATA <= X"00";
			WHEN "0011011" =>
			PKT_DATA <= X"00";	 
			WHEN "0011100" =>
			PKT_DATA <= X"00";
			WHEN "0011101" =>
			PKT_DATA <= X"00";
			WHEN "0011110" =>
			PKT_DATA <= X"80";
			WHEN "0011111" =>
			PKT_DATA <= X"11";
			
			WHEN "0100000" =>
			PKT_DATA <= IPCHECKSUM3(15 DOWNTO 8);
			WHEN "0100001" =>
			PKT_DATA <= IPCHECKSUM3(7 DOWNTO 0);
			
			WHEN "0100010" =>
			PKT_DATA <= IPSOURCE_1;
			WHEN "0100011" =>
			PKT_DATA <= IPSOURCE_2;
			WHEN "0100100" =>
			PKT_DATA <= IPSOURCE_3;
			WHEN "0100101" =>
			PKT_DATA <= IPSOURCE_4;
			
			WHEN "0100110" =>
			PKT_DATA <= IPDESTINATION_1;
			WHEN "0100111" =>
		   	PKT_DATA <= IPDESTINATION_2;
			WHEN "0101000" =>
			PKT_DATA <= IPDESTINATION_3;
			WHEN "0101001" =>
			PKT_DATA <= IPDESTINATION_4; 
			
			--UDP HEADER
			WHEN "0101010" =>
			PKT_DATA <= X"04";
			WHEN "0101011" =>
			PKT_DATA <= X"00";
			WHEN "0101100" =>
			PKT_DATA <= X"04";
			WHEN "0101101" =>
			PKT_DATA <= X"00";
			WHEN "0101110" =>
			PKT_DATA <= X"00";
			WHEN "0101111" =>
			PKT_DATA <= X"1A";
			WHEN "0110000" =>
			PKT_DATA <= X"00";
			WHEN "0110001" =>
			PKT_DATA <= X"00";	
			
			--PAYLOAD
			WHEN "0110010" =>
			PKT_DATA <= X"FE";
			WHEN "0110011" =>
			PKT_DATA <= X"ED";
			WHEN "0110100" =>
			PKT_DATA <= X"BE";
			WHEN "0110101" =>
			PKT_DATA <= X"EF";
			WHEN "0110110" =>
			PKT_DATA <= X"00";
			WHEN "0110111" =>
			PKT_DATA <= X"01";
			WHEN "0111000" =>
			PKT_DATA <= X"02";
			WHEN "0111001" =>
			PKT_DATA <= X"03";
			WHEN "0111010" =>
			PKT_DATA <= X"04";
			WHEN "0111011" =>
			PKT_DATA <= X"05";
			WHEN "0111100" =>
			PKT_DATA <= X"06";
			WHEN "0111101" =>
			PKT_DATA <= X"07";
			WHEN "0111110" =>
			PKT_DATA <= X"08";
			WHEN "0111111" =>
			PKT_DATA <= X"09";
			WHEN "1000000" =>
			PKT_DATA <= X"10";
			WHEN "1000001" =>
			PKT_DATA <= X"20";
			WHEN "1000010" =>
			PKT_DATA <= X"30";
			WHEN "1000011" =>
			PKT_DATA <= X"40";
			WHEN OTHERS => 
			PKT_DATA <= X"00";
		END CASE;
		END IF;
	END IF;
END PROCESS;
--And now, the 10BaseT magic...

--Create SENDINGPACKET flag
PROCESS (CLOCK_R,CLEAR_L)
BEGIN
	IF CLEAR_L = '0' THEN
		SENDINGPACKET <= '0';
	ELSIF RISING_EDGE (CLOCK_R) THEN 
		IF (STARTSENDING = '1') THEN
				SENDINGPACKET <= '1';
		ELSE
			IF EN_H = '1' THEN
				IF SHIFTCOUNT = "1110" AND RDADDRESS = "1001000" THEN
					SENDINGPACKET <= '0';
				END IF;
			END IF;
		END IF;
	END IF;
END PROCESS; 

TEMP_XHDL3 <= SHIFTCOUNT + "0001" WHEN SENDINGPACKET = '1' 
ELSE "1111";

PROCESS (CLOCK_R,CLEAR_L)
BEGIN
	IF CLEAR_L = '0' THEN
	SHIFTCOUNT	<= (OTHERS => '0');
	
	ELSIF RISING_EDGE (CLOCK_R) THEN 
		IF EN_H = '1' THEN
		SHIFTCOUNT <= TEMP_XHDL3;
	END IF;
	END IF;
END PROCESS;

READRAM <= '1' WHEN SHIFTCOUNT = "1111" ELSE '0';
 
--ram address counter	
PROCESS (CLOCK_R,CLEAR_L)
BEGIN
	IF CLEAR_L = '0' THEN
		RDADDRESS <= (OTHERS => '0');
	ELSIF RISING_EDGE (CLOCK_R) THEN
		IF EN_H = '1' THEN
		IF SHIFTCOUNT = "1111" THEN
			IF SENDINGPACKET = '1' THEN
			RDADDRESS <= RDADDRESS + "0000001";  
			ELSE
			RDADDRESS <= "0000000"; 
			END IF;
		END IF;	
		END IF;
	END IF;
END PROCESS;

PROCESS (CLOCK_R,CLEAR_L)
BEGIN
	IF CLEAR_L = '0' THEN
		SHIFTDATA <= (OTHERS => '0');
	ELSIF RISING_EDGE (CLOCK_R) THEN 
		IF EN_H = '1' THEN
		IF SHIFTCOUNT(0) = '1' THEN
			IF READRAM = '1' THEN
			SHIFTDATA <= PKT_DATA;
			ELSE 
			SHIFTDATA <= '0' & SHIFTDATA(7 DOWNTO 1);
			END IF;
		END IF;
		END IF;
	END IF;
END PROCESS;

--generate CRC32
PROCESS (CLOCK_R,CLEAR_L)
BEGIN
	IF CLEAR_L = '0' THEN 
		CRCFLUSH <= '0';
	ELSIF RISING_EDGE (CLOCK_R) THEN 
		IF EN_H = '1' THEN
		IF CRCFLUSH = '1' THEN
			CRCFLUSH <= SENDINGPACKET;
		ELSE
			IF READRAM = '1' THEN
				IF RDADDRESS = "1000100" THEN
					CRCFLUSH <= '1';
				ELSE 
					CRCFLUSH <= '0';
				END IF;
			END IF;
		END IF;
		END IF;
	END IF;
END PROCESS;
		
PROCESS (CLOCK_R,CLEAR_L)
BEGIN
	IF CLEAR_L = '0' THEN
		CRCINIT <= '0';
	ELSIF RISING_EDGE (CLOCK_R) THEN  
		IF EN_H = '1' THEN
		IF READRAM = '1' THEN
			IF RDADDRESS = "0000111" THEN
				CRCINIT <= '1';
			ELSE
				CRCINIT <= '0';
			END IF;
		END IF;
		END IF;
	END IF;
END PROCESS; 

TEMP_XHDL6 <= '0' WHEN CRCFLUSH = '1' ELSE (SHIFTDATA(0) XOR CRC(31));
CRCINPUT <= TEMP_XHDL6;
CRCINPUTVEC <= (CRCINPUT & CRCINPUT & CRCINPUT & 
	CRCINPUT & CRCINPUT & CRCINPUT & CRCINPUT & CRCINPUT & CRCINPUT & 
	CRCINPUT & CRCINPUT & CRCINPUT & CRCINPUT & CRCINPUT & CRCINPUT & 
	CRCINPUT & CRCINPUT & CRCINPUT & CRCINPUT & CRCINPUT & CRCINPUT & 
	CRCINPUT & CRCINPUT & CRCINPUT & CRCINPUT & CRCINPUT & CRCINPUT & 
	CRCINPUT & CRCINPUT & CRCINPUT & CRCINPUT & CRCINPUT);
CRCINPUTVEC_AND_X04C11DB7 <= CRCINPUTVEC 
	AND	X"04C11DB7";
TEMP_XHDL7 <= "11111111111111111111111111111111" WHEN CRCINIT = '1'
ELSE CRC(30 DOWNTO 0)& '0' XOR CRCINPUTVEC_AND_X04C11DB7;

PROCESS (CLOCK_R,CLEAR_L)
BEGIN
	IF CLEAR_L = '0' THEN
		CRC <= (OTHERS => '0');
	ELSIF RISING_EDGE (CLOCK_R) THEN
		IF EN_H = '1' THEN
		IF SHIFTCOUNT(0) = '1' THEN
			CRC <= TEMP_XHDL7;
		END IF;
		END IF;
	END IF;
END PROCESS;

--GENERATE THE NLP
TEMP_XHDL8 <= "000000000000000000" WHEN SENDINGPACKET = '1' ELSE
LINKPULSECOUNT + "000000000000000001";	
PROCESS (CLOCK_R,CLEAR_L)
BEGIN
	IF CLEAR_L = '0' THEN
		LINKPULSECOUNT <= (OTHERS => '0');
	ELSIF RISING_EDGE (CLOCK_R) THEN 
		IF EN_H = '1' THEN
		LINKPULSECOUNT <= TEMP_XHDL8;
		END IF;
	END IF;
END PROCESS;

PROCESS (CLOCK_R,CLEAR_L)
BEGIN
	IF CLEAR_L = '0' THEN 
		LINKPULSE <= '0';
	ELSIF RISING_EDGE (CLOCK_R) THEN 
		IF EN_H = '1' THEN
		IF LINKPULSECOUNT (17 DOWNTO 1) = "11111111111111111" THEN
		LINKPULSE <= '1';
		ELSE
		LINKPULSE <= '0';
		END IF;
		END IF;
	END IF;
END PROCESS; 

--TP_IDL, SHIFT REGISTER AND MANCHESTER ENCODER
PROCESS (CLOCK_R,CLEAR_L)
BEGIN
	IF CLEAR_L = '0' THEN
		SENDINGPACKETDATA <= '0';
	ELSIF RISING_EDGE (CLOCK_R) THEN
		IF EN_H = '1' THEN
		SENDINGPACKETDATA <= SENDINGPACKET;
		END IF;
	END IF;
END PROCESS;

PROCESS (CLOCK_R,CLEAR_L)
BEGIN
	IF CLEAR_L = '0' THEN 
		IDLECOUNT <= (OTHERS => '0');
	ELSIF RISING_EDGE (CLOCK_R) THEN  
		IF EN_H = '1' THEN
		IF SENDINGPACKETDATA = '1' THEN
			IDLECOUNT <= "000";
		ELSE
			IF IDLECOUNT /= "111" THEN
				IDLECOUNT <= IDLECOUNT + "001";
			END IF;
		END IF;
		END IF;
	END IF;
END PROCESS;

TEMP_XHDL9 <= NOT CRC(31) WHEN CRCFLUSH = '1' ELSE SHIFTDATA(0);
DATAOUT <= TEMP_XHDL9;
TEMP_XHDL10 <= ((NOT DATAOUT) XOR SHIFTCOUNT(0)) WHEN SENDINGPACKETDATA
= '1' ELSE '1';
	
PROCESS (CLOCK_R,CLEAR_L)
BEGIN
	IF CLEAR_L = '0' THEN 
		QO <= '0';
	ELSIF RISING_EDGE (CLOCK_R) THEN 
		IF EN_H = '1' THEN
		QO <= TEMP_XHDL10;
		END IF;
	END IF;
END PROCESS;

TEMPRANDY1 <= '1' WHEN IDLECOUNT < "110" ELSE '0';
PROCESS (CLOCK_R,CLEAR_L)
BEGIN
	IF CLEAR_L = '0' THEN
		QOE <= '0';
	ELSIF RISING_EDGE (CLOCK_R) THEN
		IF EN_H = '1' THEN
		QOE <= SENDINGPACKETDATA OR LINKPULSE OR 
		TEMPRANDY1;
		END IF;
	END IF;
END PROCESS;

TEMP_XHDL11 <= QO WHEN QOE = '1' ELSE '0';	
PROCESS (CLOCK_R,CLEAR_L)
BEGIN
	IF CLEAR_L = '0' THEN
		Ethernet_TDp_xhdl1 <= '0';
	ELSIF RISING_EDGE (CLOCK_R) THEN
		IF EN_H = '1' THEN
		Ethernet_TDp_xhdl1 <= TEMP_XHDL11;
		END IF;
	END IF;
END PROCESS;

TEMP_XHDL12 <= NOT QO WHEN QOE = '1' ELSE '0';
PROCESS (CLOCK_R,CLEAR_L)
BEGIN
	IF CLEAR_L = '0' THEN
		Ethernet_TDm_xhdl2 <= '0';
	ELSIF RISING_EDGE (CLOCK_R) THEN 
		IF EN_H = '1' THEN
		Ethernet_TDm_xhdl2 <= TEMP_XHDL12;
		END IF;
	END IF;
END PROCESS;
end TENBASET_TXD_RANDY_SEND_INPUT;
